@@ -17,7 +17,6 @@ use ibc::core::ics03_connection::connection::{ConnectionEnd, IdentifiedConnectio
 use ibc::core::ics04_channel::channel::{ChannelEnd, IdentifiedChannelEnd};
 use ibc::core::ics04_channel::context::ChannelReader;
 use ibc::core::ics04_channel::packet::{PacketMsgType, Sequence};
-use ibc::core::ics04_channel::Version;
 use ibc::core::ics23_commitment::{commitment::CommitmentPrefix, specs::ProofSpecs};
 use ibc::core::ics24_host::identifier::{ChainId, ChannelId, ClientId, ConnectionId, PortId};
 use ibc::events::IbcEvent;
@@ -40,7 +39,8 @@ use ibc_proto::ibc::core::connection::v1::{
     QueryClientConnectionsRequest, QueryConnectionsRequest,
 };
 
-use crate::chain::{handle::requests::AppVersion, ChainEndpoint, StatusResponse};
+use crate::chain::client::ClientSettings;
+use crate::chain::{ChainEndpoint, ChainStatus};
 use crate::config::ChainConfig;
 use crate::error::Error;
 use crate::event::monitor::{EventReceiver, EventSender, TxMonitorCmd};
@@ -162,12 +162,16 @@ impl ChainEndpoint for MockChain {
         unimplemented!()
     }
 
+    fn ibc_version(&self) -> Result<Option<semver::Version>, Error> {
+        Ok(Some(semver::Version::new(3, 0, 0)))
+    }
+
     fn query_commitment_prefix(&self) -> Result<CommitmentPrefix, Error> {
         unimplemented!()
     }
 
-    fn query_status(&self) -> Result<StatusResponse, Error> {
-        Ok(StatusResponse {
+    fn query_status(&self) -> Result<ChainStatus, Error> {
+        Ok(ChainStatus {
             height: self.context.host_height(),
             timestamp: self.context.host_timestamp(),
         })
@@ -299,6 +303,10 @@ impl ChainEndpoint for MockChain {
         unimplemented!()
     }
 
+    fn query_host_consensus_state(&self, _height: Height) -> Result<Self::ConsensusState, Error> {
+        unimplemented!()
+    }
+
     fn proven_client_state(
         &self,
         _client_id: &ClientId,
@@ -347,15 +355,19 @@ impl ChainEndpoint for MockChain {
     fn build_client_state(
         &self,
         height: Height,
-        dst_config: ChainConfig,
+        settings: ClientSettings,
     ) -> Result<Self::ClientState, Error> {
+        let ClientSettings::Tendermint(settings) = settings;
+        let trusting_period = settings
+            .trusting_period
+            .unwrap_or_else(|| self.trusting_period());
+
         let client_state = TendermintClientState::new(
             self.id().clone(),
-            self.config.trust_threshold.into(),
-            self.trusting_period(),
+            settings.trust_threshold,
+            trusting_period,
             self.trusting_period().add(Duration::from_secs(1000)),
-            // See `calculate_client_state_drift`
-            self.config.clock_drift + dst_config.clock_drift + dst_config.max_block_time,
+            settings.max_clock_drift,
             height,
             ProofSpecs::default(),
             vec!["upgrade/upgradedClient".to_string()],
@@ -437,10 +449,6 @@ impl ChainEndpoint for MockChain {
     ) -> Result<(AnyConsensusState, MerkleProof), Error> {
         unimplemented!()
     }
-
-    fn query_app_version(&self, _request: AppVersion) -> Result<Version, Error> {
-        todo!()
-    }
 }
 
 // For integration tests with the modules
