@@ -201,7 +201,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Channel<ChainA, ChainB> {
             .ok_or_else(|| ChannelError::invalid_event(channel_open_event))?;
 
         let port_id = channel_event_attributes.port_id.clone();
-        let channel_id = channel_event_attributes.channel_id.clone();
+        let channel_id = channel_event_attributes.channel_id;
 
         let connection_id = channel_event_attributes.connection_id.clone();
         let connection = chain
@@ -255,7 +255,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Channel<ChainA, ChainB> {
 
         let a_connection_id = a_channel.connection_hops().first().ok_or_else(|| {
             ChannelError::supervisor(SupervisorError::missing_connection_hops(
-                channel.src_channel_id.clone(),
+                channel.src_channel_id,
                 chain.id(),
             ))
         })?;
@@ -270,7 +270,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Channel<ChainA, ChainB> {
             .cloned()
             .ok_or_else(|| {
                 ChannelError::supervisor(SupervisorError::channel_connection_uninitialized(
-                    channel.src_channel_id.clone(),
+                    channel.src_channel_id,
                     chain.id(),
                     a_connection.counterparty().clone(),
                 ))
@@ -283,7 +283,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Channel<ChainA, ChainB> {
                 a_connection.client_id().clone(),
                 a_connection_id.clone(),
                 channel.src_port_id.clone(),
-                Some(channel.src_channel_id.clone()),
+                Some(channel.src_channel_id),
                 None,
             ),
             b_side: ChannelSide::new(
@@ -291,7 +291,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Channel<ChainA, ChainB> {
                 a_connection.counterparty().client_id().clone(),
                 b_connection_id.clone(),
                 a_channel.remote.port_id.clone(),
-                a_channel.remote.channel_id.clone(),
+                a_channel.remote.channel_id,
                 None,
             ),
             connection_delay: a_connection.delay_period(),
@@ -383,7 +383,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Channel<ChainA, ChainB> {
         info!("done {} => {:#?}\n", self.src_chain().id(), event);
 
         let channel_id = extract_channel_id(&event)?;
-        self.a_side.channel_id = Some(channel_id.clone());
+        self.a_side.channel_id = Some(*channel_id);
         info!("successfully opened init channel");
 
         Ok(())
@@ -413,7 +413,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Channel<ChainA, ChainB> {
         })?;
 
         let channel_id = extract_channel_id(&event)?;
-        self.b_side.channel_id = Some(channel_id.clone());
+        self.b_side.channel_id = Some(*channel_id);
 
         println!("done {} => {:#?}\n", self.dst_chain().id(), event);
         Ok(())
@@ -481,7 +481,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Channel<ChainA, ChainB> {
                 .map_err(|e| {
                     ChannelError::handshake_finalize(
                         channel.src_port_id().clone(),
-                        src_channel_id.clone(),
+                        *src_channel_id,
                         channel.src_chain().id(),
                         e,
                     )
@@ -493,7 +493,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Channel<ChainA, ChainB> {
                 .map_err(|e| {
                     ChannelError::handshake_finalize(
                         channel.dst_port_id().clone(),
-                        dst_channel_id.clone(),
+                        *dst_channel_id,
                         channel.dst_chain().id(),
                         e,
                     )
@@ -632,14 +632,14 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Channel<ChainA, ChainB> {
 
         let channel_deps =
             channel_connection_client(self.src_chain(), self.src_port_id(), channel_id)
-                .map_err(|e| ChannelError::query_channel(channel_id.clone(), e))?;
+                .map_err(|e| ChannelError::query_channel(*channel_id, e))?;
 
         channel_state_on_destination(
             &channel_deps.channel,
             &channel_deps.connection,
             self.dst_chain(),
         )
-        .map_err(|e| ChannelError::query_channel(channel_id.clone(), e))
+        .map_err(|e| ChannelError::query_channel(*channel_id, e))
     }
 
     pub fn handshake_step(
@@ -833,7 +833,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Channel<ChainA, ChainB> {
         }
 
         check_destination_channel_state(
-            dst_channel_id.clone(),
+            *dst_channel_id,
             dst_channel,
             dst_expected_channel.clone(),
         )?;
@@ -859,7 +859,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Channel<ChainA, ChainB> {
                 self.dst_port_id().clone(),
                 self.src_chain().id(),
                 src_channel.counterparty().port_id.clone(),
-                src_channel_id.clone(),
+                *src_channel_id,
             ));
         }
 
@@ -902,9 +902,9 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Channel<ChainA, ChainB> {
             .map_err(|e| ChannelError::fetch_signer(self.dst_chain().id(), e))?;
 
         let previous_channel_id = if src_channel.counterparty().channel_id.is_none() {
-            self.b_side.channel_id.clone()
+            self.b_side.channel_id
         } else {
-            src_channel.counterparty().channel_id.clone()
+            src_channel.counterparty().channel_id
         };
 
         // Build the domain type message
@@ -994,8 +994,8 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Channel<ChainA, ChainB> {
         // Build the domain type message
         let new_msg = MsgChannelOpenAck {
             port_id: self.dst_port_id().clone(),
-            channel_id: dst_channel_id.clone(),
-            counterparty_channel_id: src_channel_id.clone(),
+            channel_id: *dst_channel_id,
+            counterparty_channel_id: *src_channel_id,
             counterparty_version: src_channel.version().clone(),
             proofs,
             signer,
@@ -1094,7 +1094,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Channel<ChainA, ChainB> {
         // Build the domain type message
         let new_msg = MsgChannelOpenConfirm {
             port_id: self.dst_port_id().clone(),
-            channel_id: dst_channel_id.clone(),
+            channel_id: *dst_channel_id,
             proofs,
             signer,
         };
@@ -1163,7 +1163,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Channel<ChainA, ChainB> {
         // Build the domain type message
         let new_msg = MsgChannelCloseInit {
             port_id: self.dst_port_id().clone(),
-            channel_id: dst_channel_id.clone(),
+            channel_id: *dst_channel_id,
             signer,
         };
 
@@ -1242,7 +1242,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Channel<ChainA, ChainB> {
         // Build the domain type message
         let new_msg = MsgChannelCloseConfirm {
             port_id: self.dst_port_id().clone(),
-            channel_id: dst_channel_id.clone(),
+            channel_id: *dst_channel_id,
             proofs,
             signer,
         };
