@@ -4,19 +4,18 @@ use ibc::{
         ics04_channel::channel::State as ChannelState,
         ics24_host::identifier::{ChannelId, PortChannelId, PortId},
     },
-    events::IbcEvent,
     Height,
 };
-use tracing::error_span;
 
 use crate::chain::counterparty::check_channel_counterparty;
 use crate::chain::handle::ChainHandle;
 use crate::channel::{Channel, ChannelSide};
 use crate::link::error::LinkError;
-use crate::link::relay_path::RelayPath;
 
+pub mod cli;
 pub mod error;
 pub mod operational_data;
+
 mod pending;
 mod relay_path;
 mod relay_sender;
@@ -27,6 +26,8 @@ use tx_hashes::TxHashes;
 // Re-export the telemetries summary
 pub use relay_summary::RelaySummary;
 
+pub use relay_path::{RelayPath, Resubmit};
+
 #[derive(Clone, Debug)]
 pub struct LinkParameters {
     pub src_port_id: PortId,
@@ -57,7 +58,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Link<ChainA, ChainB> {
             .map_err(|e| {
                 LinkError::channel_not_found(
                     self.a_to_b.src_port_id().clone(),
-                    a_channel_id.clone(),
+                    *a_channel_id,
                     self.a_to_b.src_chain().id(),
                     e,
                 )
@@ -72,7 +73,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Link<ChainA, ChainB> {
             .map_err(|e| {
                 LinkError::channel_not_found(
                     self.a_to_b.dst_port_id().clone(),
-                    b_channel_id.clone(),
+                    *b_channel_id,
                     self.a_to_b.dst_chain().id(),
                     e,
                 )
@@ -99,19 +100,14 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Link<ChainA, ChainB> {
         let a_channel = a_chain
             .query_channel(a_port_id, a_channel_id, Height::default())
             .map_err(|e| {
-                LinkError::channel_not_found(
-                    a_port_id.clone(),
-                    a_channel_id.clone(),
-                    a_chain.id(),
-                    e,
-                )
+                LinkError::channel_not_found(a_port_id.clone(), *a_channel_id, a_chain.id(), e)
             })?;
 
         if !a_channel.state_matches(&ChannelState::Open)
             && !a_channel.state_matches(&ChannelState::Closed)
         {
             return Err(LinkError::invalid_channel_state(
-                a_channel_id.clone(),
+                *a_channel_id,
                 a_chain.id(),
             ));
         }
@@ -119,25 +115,21 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Link<ChainA, ChainB> {
         let b_channel_id = a_channel
             .counterparty()
             .channel_id
-            .clone()
-            .ok_or_else(|| LinkError::counterparty_channel_not_found(a_channel_id.clone()))?;
+            .ok_or_else(|| LinkError::counterparty_channel_not_found(*a_channel_id))?;
 
         if a_channel.connection_hops().is_empty() {
-            return Err(LinkError::no_connection_hop(
-                a_channel_id.clone(),
-                a_chain.id(),
-            ));
+            return Err(LinkError::no_connection_hop(*a_channel_id, a_chain.id()));
         }
 
         // Check that the counterparty details on the destination chain matches the source chain
         check_channel_counterparty(
             b_chain.clone(),
             &PortChannelId {
-                channel_id: b_channel_id.clone(),
+                channel_id: b_channel_id,
                 port_id: a_channel.counterparty().port_id.clone(),
             },
             &PortChannelId {
-                channel_id: a_channel_id.clone(),
+                channel_id: *a_channel_id,
                 port_id: opts.src_port_id.clone(),
             },
         )
@@ -150,10 +142,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Link<ChainA, ChainB> {
             .map_err(LinkError::relayer)?;
 
         if !a_connection.state_matches(&ConnectionState::Open) {
-            return Err(LinkError::channel_not_opened(
-                a_channel_id.clone(),
-                a_chain.id(),
-            ));
+            return Err(LinkError::channel_not_opened(*a_channel_id, a_chain.id()));
         }
 
         let channel = Channel {
@@ -163,7 +152,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Link<ChainA, ChainB> {
                 a_connection.client_id().clone(),
                 a_connection_id,
                 opts.src_port_id.clone(),
-                Some(opts.src_channel_id.clone()),
+                Some(opts.src_channel_id),
                 None,
             ),
             b_side: ChannelSide::new(
@@ -185,7 +174,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Link<ChainA, ChainB> {
     pub fn reverse(&self, with_tx_confirmation: bool) -> Result<Link<ChainB, ChainA>, LinkError> {
         let opts = LinkParameters {
             src_port_id: self.a_to_b.dst_port_id().clone(),
-            src_channel_id: self.a_to_b.dst_channel_id().clone(),
+            src_channel_id: *self.a_to_b.dst_channel_id(),
         };
         let chain_b = self.a_to_b.dst_chain().clone();
         let chain_a = self.a_to_b.src_chain().clone();
@@ -194,56 +183,4 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> Link<ChainA, ChainB> {
         // going slowly, but reliably.
         Link::new_from_opts(chain_b, chain_a, opts, with_tx_confirmation)
     }
-
-    /// Implements the `packet-recv` CLI
-    pub fn build_and_send_recv_packet_messages(&self) -> Result<Vec<IbcEvent>, LinkError> {
-        let _span = error_span!(
-            "PacketRecvCmd",
-            src_chain = %self.a_to_b.src_chain().id(),
-            src_port = %self.a_to_b.src_port_id(),
-            src_channel = %self.a_to_b.src_channel_id(),
-            dst_chain = %self.a_to_b.dst_chain().id(),
-        )
-        .entered();
-
-        self.a_to_b.build_recv_packet_and_timeout_msgs(None)?;
-
-        let mut results = vec![];
-
-        // Block waiting for all of the scheduled data (until `None` is returned)
-        while let Some(odata) = self.a_to_b.fetch_scheduled_operational_data() {
-            let mut last_res = self
-                .a_to_b
-                .relay_from_operational_data::<relay_sender::SyncSender>(odata)?;
-            results.append(&mut last_res.events);
-        }
-
-        Ok(results)
-    }
-
-    /// Implements the `packet-ack` CLI
-    pub fn build_and_send_ack_packet_messages(&self) -> Result<Vec<IbcEvent>, LinkError> {
-        let _span = error_span!(
-            "PacketAckCmd",
-            src_chain = %self.a_to_b.src_chain().id(),
-            src_port = %self.a_to_b.src_port_id(),
-            src_channel = %self.a_to_b.src_channel_id(),
-            dst_chain = %self.a_to_b.dst_chain().id(),
-        )
-        .entered();
-
-        self.a_to_b.build_packet_ack_msgs(None)?;
-
-        let mut results = vec![];
-
-        // Block waiting for all of the scheduled data
-        while let Some(odata) = self.a_to_b.fetch_scheduled_operational_data() {
-            let mut last_res = self
-                .a_to_b
-                .relay_from_operational_data::<relay_sender::SyncSender>(odata)?;
-            results.append(&mut last_res.events);
-        }
-
-        Ok(results)
-    }
 }
