@@ -1,23 +1,22 @@
-use core::fmt;
 use core::iter::Iterator;
 use core::time::Duration;
 use std::time::Instant;
 
-use tracing::{debug, error, trace};
+use tracing::{debug, error, trace, trace_span};
 
 use ibc::core::ics24_host::identifier::{ChainId, ChannelId, PortId};
 use ibc::events::IbcEvent;
 use ibc::query::{QueryTxHash, QueryTxRequest};
 
 use crate::error::Error as RelayerError;
-use crate::link::error::LinkError;
+use crate::link::{error::LinkError, RelayPath};
 use crate::util::queue::Queue;
 use crate::{
     chain::handle::ChainHandle,
     link::{operational_data::OperationalData, relay_sender::AsyncReply, RelaySummary, TxHashes},
 };
 
-pub const TIMEOUT: Duration = Duration::from_secs(100);
+pub const TIMEOUT: Duration = Duration::from_secs(300);
 
 /// A wrapper over an [`OperationalData`] that is pending.
 /// Additionally holds all the necessary information
@@ -66,7 +65,7 @@ impl<Chain: ChainHandle> PendingTxs<Chain> {
         self.chain.id()
     }
 
-    // Insert new pending transaction to the back of the queue.
+    /// Insert a new pending transaction to the back of the queue.
     pub fn insert_new_pending_tx(&self, r: AsyncReply, od: OperationalData) {
         let mut tx_hashes = Vec::new();
         let mut error_events = Vec::new();
@@ -80,9 +79,19 @@ impl<Chain: ChainHandle> PendingTxs<Chain> {
                 // transactions have been confirmed.
                 // This is not ideal, but is just to follow the previous synchronous
                 // behavior of handling the OperationalData.
+
+                let span = trace_span!(
+                    "inserting new pending txs",
+                    chain = %self.chain_id(),
+                    counterparty_chain = %self.counterparty_chain_id,
+                    port = %self.port_id,
+                    channel = %self.channel_id,
+                );
+
+                let _guard = span.enter();
+
                 trace!(
-                    "[{}] putting error response in error event queue: {:?} ",
-                    self,
+                    "putting error response in error event queue: {:?} ",
                     response
                 );
 
@@ -124,11 +133,17 @@ impl<Chain: ChainHandle> PendingTxs<Chain> {
         Ok(Some(all_events))
     }
 
-    // Try and process one pending transaction if available.
-    pub fn process_pending(
+    /// Try and process one pending transaction within the given timeout duration if one
+    /// is available.
+    ///
+    /// A `resubmit` closure is provided when the clear interval for packets is 0. If this closure
+    /// is provided, the pending transactions that fail to process within the given timeout duration
+    /// are resubmitted following the logic specified by the closure.
+    pub fn process_pending<ChainA: ChainHandle, ChainB: ChainHandle>(
         &self,
         timeout: Duration,
-        resubmit: impl FnOnce(OperationalData) -> Result<AsyncReply, LinkError>,
+        relay_path: &RelayPath<ChainA, ChainB>,
+        resubmit: Option<impl FnOnce(OperationalData) -> Result<AsyncReply, LinkError>>,
     ) -> Result<Option<RelaySummary>, LinkError> {
         // We process pending transactions in a FIFO manner, so take from
         // the front of the queue.
@@ -140,8 +155,18 @@ impl<Chain: ChainHandle> PendingTxs<Chain> {
                 return Ok(Some(RelaySummary::from_events(pending.error_events)));
             }
 
+            let span = trace_span!(
+                "processing pending tx",
+                chain = %self.chain_id(),
+                counterparty_chain = %self.counterparty_chain_id,
+                port = %self.port_id,
+                channel = %self.channel_id,
+            );
+
+            let _guard = span.enter();
+
             // Process the given pending transaction.
-            trace!("[{}] trying to confirm {} ", self, tx_hashes);
+            trace!("trying to confirm {} ", tx_hashes);
 
             // Check for TX events for the given pending transaction hashes.
             let events_result = self.check_tx_events(tx_hashes);
@@ -150,28 +175,42 @@ impl<Chain: ChainHandle> PendingTxs<Chain> {
                     // There is no events for the associated transactions.
                     // This means the transaction has not yet been committed.
 
-                    trace!(
-                        "[{}] transaction is not yet committed: {} ",
-                        self,
-                        tx_hashes
-                    );
+                    trace!("transaction is not yet committed: {} ", tx_hashes);
 
                     if submit_time.elapsed() > timeout {
                         // The submission time for the transaction has exceeded the
                         // timeout threshold. Returning Outcome::Timeout for the
                         // relayer to resubmit the transaction to the chain again.
-                        error!("[{}] timed out while confirming {}", self, tx_hashes);
+                        error!("timed out while confirming {}", tx_hashes);
 
-                        let resubmit_res = resubmit(pending.original_od.clone());
+                        match resubmit {
+                            Some(f) => {
+                                // The pending tx needs to be resubmitted. This involves replacing the tx's
+                                // stale operational data with a fresh copy and then applying the `resubmit`
+                                // closure to it.
+                                let new_od = relay_path
+                                    .regenerate_operational_data(pending.original_od.clone());
 
-                        match resubmit_res {
-                            Ok(reply) => {
-                                self.insert_new_pending_tx(reply, pending.original_od);
-                                Ok(None)
+                                trace!("regenerated operational data for {}", tx_hashes);
+
+                                match new_od.map(f) {
+                                    Some(Ok(reply)) => {
+                                        self.insert_new_pending_tx(reply, pending.original_od);
+                                        Ok(None)
+                                    }
+                                    Some(Err(e)) => {
+                                        self.pending_queue.push_back(pending);
+                                        Err(e)
+                                    }
+                                    None => {
+                                        // No operational data was regenerated; nothing to resubmit
+                                        Ok(None)
+                                    }
+                                }
                             }
-                            Err(e) => {
-                                self.pending_queue.push_back(pending);
-                                Err(e)
+                            None => {
+                                // `clear_interval != 0` such that resubmission has been disabled
+                                Ok(None)
                             }
                         }
                     } else {
@@ -188,8 +227,7 @@ impl<Chain: ChainHandle> PendingTxs<Chain> {
                     // to the chain.
 
                     debug!(
-                        "[{}] confirmed after {:#?}: {} ",
-                        self,
+                        "confirmed after {:#?}: {} ",
                         pending.submit_time.elapsed(),
                         tx_hashes
                     );
@@ -206,8 +244,8 @@ impl<Chain: ChainHandle> PendingTxs<Chain> {
                     // with the chain endpoint.
 
                     error!(
-                        "[{}] error querying for tx hashes {}: {}. will retry again later",
-                        self, tx_hashes, e
+                        "error querying for tx hashes {}: {}. will retry again later",
+                        tx_hashes, e
                     );
 
                     // Push it to the back of the pending queue to process it
@@ -220,8 +258,7 @@ impl<Chain: ChainHandle> PendingTxs<Chain> {
 
             if !self.pending_queue.is_empty() {
                 trace!(
-                    "[{}] total pending transactions left: {}",
-                    self,
+                    "total pending transactions left: {}",
                     self.pending_queue.len()
                 );
             }
@@ -232,16 +269,3 @@ impl<Chain: ChainHandle> PendingTxs<Chain> {
         }
     }
 }
-
-impl<Chain: ChainHandle> fmt::Display for PendingTxs<Chain> {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        write!(
-            f,
-            "{}:{}/{} -> {}",
-            self.chain_id(),
-            self.port_id,
-            self.channel_id,
-            self.counterparty_chain_id
-        )
-    }
-}
