@@ -1,7 +1,7 @@
 use alloc::collections::BTreeMap as HashMap;
 use alloc::collections::VecDeque;
-use std::thread;
-use std::time::Instant;
+use std::ops::Sub;
+use std::time::{Duration, Instant};
 
 use ibc_proto::google::protobuf::Any;
 use itertools::Itertools;
@@ -9,6 +9,11 @@ use tracing::{debug, error, info, span, trace, warn, Level};
 
 use ibc::{
     core::{
+        ics02_client::{
+            client_consensus::QueryClientEventRequest,
+            events::ClientMisbehaviour as ClientMisbehaviourEvent,
+            events::UpdateClient as UpdateClientEvent,
+        },
         ics04_channel::{
             channel::{ChannelEnd, Order, QueryPacketEventDataRequest, State as ChannelState},
             events::{SendPacket, WriteAcknowledgement},
@@ -24,7 +29,7 @@ use ibc::{
     events::{IbcEvent, PrettyEvents, WithBlockDataType},
     query::{QueryBlockRequest, QueryTxRequest},
     signer::Signer,
-    timestamp::ZERO_DURATION,
+    timestamp::Timestamp,
     tx_msg::Msg,
     Height,
 };
@@ -33,11 +38,12 @@ use ibc_proto::ibc::core::channel::v1::{
 };
 
 use crate::chain::counterparty::{
-    unreceived_acknowledgements_sequences, unreceived_packets_sequences,
+    commitments_on_chain, packet_acknowledgements, unreceived_acknowledgements_sequences,
+    unreceived_packets_sequences,
 };
 use crate::chain::handle::ChainHandle;
 use crate::chain::tx::TrackedMsgs;
-use crate::chain::StatusResponse;
+use crate::chain::ChainStatus;
 use crate::channel::error::ChannelError;
 use crate::channel::Channel;
 use crate::event::monitor::EventBatch;
@@ -54,6 +60,27 @@ use crate::util::queue::Queue;
 
 const MAX_RETRIES: usize = 5;
 
+/// Whether or not to resubmit packets when pending transactions
+/// fail to process within the given timeout duration.
+#[derive(Copy, Clone, Debug, PartialEq, Eq)]
+pub enum Resubmit {
+    Yes,
+    No,
+}
+
+impl Resubmit {
+    /// Packet resubmission is enabled when the clear interval for packets is 0. Otherwise,
+    /// when the packet clear interval is > 0, the relayer will periodically clear unsent packets
+    /// such that resubmitting packets is not necessary.
+    pub fn from_clear_interval(clear_interval: u64) -> Self {
+        if clear_interval == 0 {
+            Self::Yes
+        } else {
+            Self::No
+        }
+    }
+}
+
 pub struct RelayPath<ChainA: ChainHandle, ChainB: ChainHandle> {
     channel: Channel<ChainA, ChainB>,
 
@@ -70,8 +97,8 @@ pub struct RelayPath<ChainA: ChainHandle, ChainB: ChainHandle> {
     // mostly timeout packet messages.
     // The operational data targeting the destination chain
     // comprises mostly RecvPacket and Ack msgs.
-    src_operational_data: Queue<OperationalData>,
-    dst_operational_data: Queue<OperationalData>,
+    pub(crate) src_operational_data: Queue<OperationalData>,
+    pub(crate) dst_operational_data: Queue<OperationalData>,
 
     // Toggle for the transaction confirmation mechanism.
     confirm_txes: bool,
@@ -94,15 +121,13 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
         let src_chain_id = src_chain.id();
         let dst_chain_id = dst_chain.id();
 
-        let src_channel_id = channel
+        let src_channel_id = *channel
             .src_channel_id()
-            .ok_or_else(|| LinkError::missing_channel_id(src_chain.id()))?
-            .clone();
+            .ok_or_else(|| LinkError::missing_channel_id(src_chain.id()))?;
 
-        let dst_channel_id = channel
+        let dst_channel_id = *channel
             .dst_channel_id()
-            .ok_or_else(|| LinkError::missing_channel_id(dst_chain.id()))?
-            .clone();
+            .ok_or_else(|| LinkError::missing_channel_id(dst_chain.id()))?;
 
         let src_port_id = channel.src_port_id().clone();
         let dst_port_id = channel.dst_port_id().clone();
@@ -110,9 +135,9 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
         Ok(Self {
             channel,
 
-            src_channel_id: src_channel_id.clone(),
+            src_channel_id,
             src_port_id: src_port_id.clone(),
-            dst_channel_id: dst_channel_id.clone(),
+            dst_channel_id,
             dst_port_id: dst_port_id.clone(),
 
             src_operational_data: Queue::new(),
@@ -177,7 +202,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
     fn dst_channel(&self, height: Height) -> Result<ChannelEnd, LinkError> {
         self.dst_chain()
             .query_channel(self.dst_port_id(), self.dst_channel_id(), height)
-            .map_err(|e| LinkError::channel(ChannelError::query(self.src_chain().id(), e)))
+            .map_err(|e| LinkError::channel(ChannelError::query(self.dst_chain().id(), e)))
     }
 
     fn src_signer(&self) -> Result<Signer, LinkError> {
@@ -192,12 +217,52 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
             .map_err(|e| LinkError::signer(self.dst_chain().id(), e))
     }
 
-    pub fn dst_latest_height(&self) -> Result<Height, LinkError> {
+    pub(crate) fn src_latest_height(&self) -> Result<Height, LinkError> {
+        self.src_chain()
+            .query_latest_height()
+            .map_err(|e| LinkError::query(self.src_chain().id(), e))
+    }
+
+    pub(crate) fn dst_latest_height(&self) -> Result<Height, LinkError> {
         self.dst_chain()
             .query_latest_height()
             .map_err(|e| LinkError::query(self.dst_chain().id(), e))
     }
 
+    fn src_time_at_height(&self, height: Height) -> Result<Instant, LinkError> {
+        Self::chain_time_at_height(self.src_chain(), height)
+    }
+
+    fn dst_time_at_height(&self, height: Height) -> Result<Instant, LinkError> {
+        Self::chain_time_at_height(self.dst_chain(), height)
+    }
+
+    pub(crate) fn src_time_latest(&self) -> Result<Instant, LinkError> {
+        self.src_time_at_height(Height::zero())
+    }
+
+    pub(crate) fn dst_time_latest(&self) -> Result<Instant, LinkError> {
+        self.dst_time_at_height(Height::zero())
+    }
+
+    pub(crate) fn src_max_block_time(&self) -> Result<Duration, LinkError> {
+        // TODO(hu55a1n1): Ideally, we should get the `max_expected_time_per_block` using the
+        // `/genesis` endpoint once it is working in tendermint-rs.
+        Ok(self
+            .src_chain()
+            .config()
+            .map_err(LinkError::relayer)?
+            .max_block_time)
+    }
+
+    pub(crate) fn dst_max_block_time(&self) -> Result<Duration, LinkError> {
+        Ok(self
+            .dst_chain()
+            .config()
+            .map_err(LinkError::relayer)?
+            .max_block_time)
+    }
+
     fn unordered_channel(&self) -> bool {
         self.channel.ordering == Order::Unordered
     }
@@ -230,7 +295,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
         // Build the domain type message
         let new_msg = MsgChannelCloseConfirm {
             port_id: self.dst_port_id().clone(),
-            channel_id: self.dst_channel_id().clone(),
+            channel_id: *self.dst_channel_id(),
             proofs,
             signer: self.dst_signer()?,
         };
@@ -372,12 +437,14 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
             dst_latest_height,
             OperationalDataTarget::Source,
             events.tracking_id(),
+            self.channel.connection_delay,
         );
         // Operational data targeting the destination chain (e.g., SendPacket messages)
         let mut dst_od = OperationalData::new(
             src_height,
             OperationalDataTarget::Destination,
             events.tracking_id(),
+            self.channel.connection_delay,
         );
 
         for event in input {
@@ -479,12 +546,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
         let mut odata = initial_od;
 
         for i in 0..MAX_RETRIES {
-            debug!(
-                "delayed by: {:?} [try {}/{}]",
-                odata.scheduled_time.elapsed(),
-                i + 1,
-                MAX_RETRIES
-            );
+            debug!("[try {}/{}]", i + 1, MAX_RETRIES);
 
             // Consume the operational data by attempting to send its messages
             match self.send_from_operational_data::<S>(odata.clone()) {
@@ -517,8 +579,8 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
         Ok(S::Reply::empty())
     }
 
-    /// Helper for managing retries of the `relay_from_operational_data` method.
-    /// Expects as input the initial operational data that failed to send.
+    /// Generates fresh operational data for a tx given the initial operational data
+    /// that failed to send.
     ///
     /// Return value:
     ///   - `Some(..)`: a new operational data from which to retry sending,
@@ -527,7 +589,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
     ///
     /// Side effects: may schedule a new operational data targeting the source chain, comprising
     /// new timeout messages.
-    fn regenerate_operational_data(
+    pub(crate) fn regenerate_operational_data(
         &self,
         initial_odata: OperationalData,
     ) -> Option<OperationalData> {
@@ -693,103 +755,207 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
         self.recv_packet_acknowledged_on_src(&rp.packet)
     }
 
-    /// Returns `true` if the delay for this relaying path is zero.
-    /// Conversely, returns `false` if the delay is non-zero.
-    pub fn zero_delay(&self) -> bool {
-        self.channel.connection_delay == ZERO_DURATION
+    /// Returns the `processed_height` for the consensus state at specified height
+    fn update_height(
+        chain: &impl ChainHandle,
+        client_id: ClientId,
+        consensus_height: Height,
+    ) -> Result<Height, LinkError> {
+        let events = chain
+            .query_txs(QueryTxRequest::Client(QueryClientEventRequest {
+                height: Height::zero(),
+                event_id: WithBlockDataType::UpdateClient,
+                client_id,
+                consensus_height,
+            }))
+            .map_err(|e| LinkError::query(chain.id(), e))?;
+
+        // The handler may treat redundant updates as no-ops and emit `UpdateClient` events for them
+        // but the `processed_height` is the height at which the first `UpdateClient` event for this
+        // consensus state/height was emitted. We expect that these events are received in the exact
+        // same order in which they were emitted.
+        match events.first() {
+            Some(IbcEvent::UpdateClient(event)) => Ok(event.height()),
+            Some(event) => Err(LinkError::unexpected_event(event.clone())),
+            None => Err(LinkError::unexpected_event(IbcEvent::default())),
+        }
+    }
+
+    /// Loops over `tx_events` and returns a tuple of optional events where the first element is a
+    /// `ChainError` variant, the second one is an `UpdateClient` variant and the third one is a
+    /// `ClientMisbehaviour` variant. This function is essentially just an `Iterator::find()` for
+    /// multiple variants with a single pass.
+    #[inline]
+    fn event_per_type(
+        mut tx_events: Vec<IbcEvent>,
+    ) -> (
+        Option<IbcEvent>,
+        Option<UpdateClientEvent>,
+        Option<ClientMisbehaviourEvent>,
+    ) {
+        let mut error = None;
+        let mut update = None;
+        let mut misbehaviour = None;
+
+        while let Some(event) = tx_events.pop() {
+            match event {
+                IbcEvent::ChainError(_) => error = Some(event),
+                IbcEvent::UpdateClient(event) => update = Some(event),
+                IbcEvent::ClientMisbehaviour(event) => misbehaviour = Some(event),
+                _ => {}
+            }
+        }
+
+        (error, update, misbehaviour)
+    }
+
+    /// Returns an instant (in the past) that corresponds to the block timestamp of the chain at
+    /// specified height (relative to the relayer's current time). If the timestamp is in the future
+    /// wrt the relayer's current time, we simply return the current relayer time.
+    fn chain_time_at_height(
+        chain: &impl ChainHandle,
+        height: Height,
+    ) -> Result<Instant, LinkError> {
+        let chain_time = chain
+            .query_host_consensus_state(height)
+            .map_err(LinkError::relayer)?
+            .timestamp();
+        let duration = Timestamp::now()
+            .duration_since(&chain_time)
+            .unwrap_or_default();
+        Ok(Instant::now().sub(duration))
     }
 
     /// Handles updating the client on the destination chain
+    /// Returns the height at which the client update was processed
     fn update_client_dst(
         &self,
         src_chain_height: Height,
         tracking_id: &str,
-    ) -> Result<(), LinkError> {
-        // Handle the update on the destination chain
-        // Check if a consensus state at update_height exists on destination chain already
-        if self
+    ) -> Result<Height, LinkError> {
+        self.do_update_client_dst(src_chain_height, tracking_id, MAX_RETRIES)
+    }
+
+    /// Perform actual update_client_dst with retries.
+    ///
+    /// Note that the retry is only performed in the case when there
+    /// is a ChainError event. It would return error immediately if
+    /// there are other errors returned from calls such as
+    /// build_update_client_on_dst.
+    fn do_update_client_dst(
+        &self,
+        src_chain_height: Height,
+        tracking_id: &str,
+        retries_left: usize,
+    ) -> Result<Height, LinkError> {
+        info!( "sending update_client to client hosted on source chain for height {} (retries left: {})", src_chain_height, retries_left );
+
+        let dst_update = self.build_update_client_on_dst(src_chain_height)?;
+        let tm = TrackedMsgs::new(dst_update, tracking_id);
+        let dst_tx_events = self
             .dst_chain()
-            .proven_client_consensus(self.dst_client_id(), src_chain_height, Height::zero())
-            .is_ok()
-        {
-            return Ok(());
-        }
+            .send_messages_and_wait_commit(tm)
+            .map_err(LinkError::relayer)?;
+        info!("result: {}", PrettyEvents(&dst_tx_events));
 
-        let mut dst_err_ev = None;
-        for i in 0..MAX_RETRIES {
-            let dst_update = self.build_update_client_on_dst(src_chain_height)?;
-            info!(
-                "sending updateClient to client hosted on destination chain for height {} [try {}/{}]",
-                src_chain_height,
-                i + 1, MAX_RETRIES,
-            );
-
-            let tm = TrackedMsgs::new(dst_update, tracking_id);
-
-            let dst_tx_events = self
-                .dst_chain()
-                .send_messages_and_wait_commit(tm)
-                .map_err(LinkError::relayer)?;
-            info!("result: {}", PrettyEvents(&dst_tx_events));
-
-            dst_err_ev = dst_tx_events
-                .into_iter()
-                .find(|event| matches!(event, IbcEvent::ChainError(_)));
-
-            if dst_err_ev.is_none() {
-                return Ok(());
+        let (error, update, misbehaviour) = Self::event_per_type(dst_tx_events);
+        match (error, update, misbehaviour) {
+            // All updates were successful, no errors and no misbehaviour.
+            (None, Some(update_event), None) => Ok(update_event.height()),
+            (Some(chain_error), _, _) => {
+                // Atleast one chain-error so retry if possible.
+                if retries_left == 0 {
+                    Err(LinkError::client(ForeignClientError::chain_error_event(
+                        self.dst_chain().id(),
+                        chain_error,
+                    )))
+                } else {
+                    self.do_update_client_dst(src_chain_height, tracking_id, retries_left - 1)
+                }
             }
+            (None, None, None) => {
+                // `tm` was empty and update wasn't required
+                match Self::update_height(
+                    self.dst_chain(),
+                    self.dst_client_id().clone(),
+                    src_chain_height,
+                ) {
+                    Ok(update_height) => Ok(update_height),
+                    Err(_) if retries_left > 0 => {
+                        self.do_update_client_dst(src_chain_height, tracking_id, retries_left - 1)
+                    }
+                    _ => Err(LinkError::update_client_failed()),
+                }
+            }
+            // Atleast one misbehaviour event, so don't retry.
+            (_, _, Some(_misbehaviour)) => Err(LinkError::update_client_failed()),
         }
-
-        Err(LinkError::client(ForeignClientError::chain_error_event(
-            self.dst_chain().id(),
-            dst_err_ev.unwrap(),
-        )))
     }
 
     /// Handles updating the client on the source chain
+    /// Returns the height at which the client update was processed
     fn update_client_src(
         &self,
         dst_chain_height: Height,
         tracking_id: &str,
-    ) -> Result<(), LinkError> {
-        if self
+    ) -> Result<Height, LinkError> {
+        self.do_update_client_src(dst_chain_height, tracking_id, MAX_RETRIES)
+    }
+
+    /// Perform actual update_client_src with retries.
+    ///
+    /// Note that the retry is only performed in the case when there
+    /// is a ChainError event. It would return error immediately if
+    /// there are other errors returned from calls such as
+    /// build_update_client_on_src.
+    fn do_update_client_src(
+        &self,
+        dst_chain_height: Height,
+        tracking_id: &str,
+        retries_left: usize,
+    ) -> Result<Height, LinkError> {
+        info!( "sending update_client to client hosted on source chain for height {} (retries left: {})", dst_chain_height, retries_left );
+
+        let src_update = self.build_update_client_on_src(dst_chain_height)?;
+        let tm = TrackedMsgs::new(src_update, tracking_id);
+        let src_tx_events = self
             .src_chain()
-            .proven_client_consensus(self.src_client_id(), dst_chain_height, Height::zero())
-            .is_ok()
-        {
-            return Ok(());
-        }
+            .send_messages_and_wait_commit(tm)
+            .map_err(LinkError::relayer)?;
+        info!("result: {}", PrettyEvents(&src_tx_events));
 
-        let mut src_err_ev = None;
-        for _ in 0..MAX_RETRIES {
-            let src_update = self.build_update_client_on_src(dst_chain_height)?;
-            info!(
-                "sending updateClient to client hosted on source chain for height {}",
-                dst_chain_height,
-            );
-
-            let tm = TrackedMsgs::new(src_update, tracking_id);
-
-            let src_tx_events = self
-                .src_chain()
-                .send_messages_and_wait_commit(tm)
-                .map_err(LinkError::relayer)?;
-            info!("result: {}", PrettyEvents(&src_tx_events));
-
-            src_err_ev = src_tx_events
-                .into_iter()
-                .find(|event| matches!(event, IbcEvent::ChainError(_)));
-
-            if src_err_ev.is_none() {
-                return Ok(());
+        let (error, update, misbehaviour) = Self::event_per_type(src_tx_events);
+        match (error, update, misbehaviour) {
+            // All updates were successful, no errors and no misbehaviour.
+            (None, Some(update_event), None) => Ok(update_event.height()),
+            (Some(chain_error), _, _) => {
+                // Atleast one chain-error so retry if possible.
+                if retries_left == 0 {
+                    Err(LinkError::client(ForeignClientError::chain_error_event(
+                        self.src_chain().id(),
+                        chain_error,
+                    )))
+                } else {
+                    self.do_update_client_src(dst_chain_height, tracking_id, retries_left - 1)
+                }
             }
+            (None, None, None) => {
+                // `tm` was empty and update wasn't required
+                match Self::update_height(
+                    self.src_chain(),
+                    self.src_client_id().clone(),
+                    dst_chain_height,
+                ) {
+                    Ok(update_height) => Ok(update_height),
+                    Err(_) if retries_left > 0 => {
+                        self.do_update_client_src(dst_chain_height, tracking_id, retries_left - 1)
+                    }
+                    _ => Err(LinkError::update_client_failed()),
+                }
+            }
+            // Atleast one misbehaviour event, so don't retry.
+            (_, _, Some(_misbehaviour)) => Err(LinkError::update_client_failed()),
         }
-
-        Err(LinkError::client(ForeignClientError::chain_error_event(
-            self.src_chain().id(),
-            src_err_ev.unwrap(),
-        )))
     }
 
     /// Returns relevant packet events for building RecvPacket and timeout messages.
@@ -803,13 +969,15 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
         let src_channel_id = self.src_channel_id();
         let dst_channel_id = self.dst_channel_id();
 
-        let (commit_sequences, sequences, src_response_height) = unreceived_packets_sequences(
+        let (commit_sequences, src_response_height) =
+            commitments_on_chain(self.src_chain(), self.src_port_id(), src_channel_id)
+                .map_err(LinkError::supervisor)?;
+
+        let sequences = unreceived_packets_sequences(
             self.dst_chain(),
             self.dst_port_id(),
             dst_channel_id,
-            self.src_chain(),
-            self.src_port_id(),
-            src_channel_id,
+            commit_sequences,
         )
         .map_err(LinkError::supervisor)?;
 
@@ -820,13 +988,6 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
             return Ok((events_result.into(), query_height));
         }
 
-        debug!(
-            "packet seq. that still have commitments on {}: {} (first 10 shown here; total={})",
-            self.src_chain().id(),
-            commit_sequences.iter().take(10).join(", "),
-            commit_sequences.len()
-        );
-
         debug!(
             "recv packets to send out to {} of the ones with commitments on {}: {} (first 10 shown here; total={})",
             self.dst_chain().id(),
@@ -837,9 +998,9 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
         let mut query = QueryPacketEventDataRequest {
             event_id: WithBlockDataType::SendPacket,
             source_port_id: self.src_port_id().clone(),
-            source_channel_id: src_channel_id.clone(),
+            source_channel_id: *src_channel_id,
             destination_port_id: self.dst_port_id().clone(),
-            destination_channel_id: dst_channel_id.clone(),
+            destination_channel_id: *dst_channel_id,
             sequences,
             height: query_height,
         };
@@ -914,16 +1075,25 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
         let src_channel_id = self.src_channel_id();
         let dst_channel_id = self.dst_channel_id();
 
-        let (acks_on_src, unreceived_acks_by_dst, src_response_height) =
-            unreceived_acknowledgements_sequences(
-                self.dst_chain(),
-                self.dst_port_id(),
-                dst_channel_id,
-                self.src_chain(),
-                self.src_port_id(),
-                src_channel_id,
-            )
-            .map_err(LinkError::supervisor)?;
+        let (commitments_on_counterparty, _) =
+            commitments_on_chain(self.dst_chain(), self.dst_port_id(), dst_channel_id)
+                .map_err(LinkError::supervisor)?;
+
+        let (acks_on_src, src_response_height) = packet_acknowledgements(
+            self.src_chain(),
+            self.src_port_id(),
+            src_channel_id,
+            commitments_on_counterparty,
+        )
+        .map_err(LinkError::supervisor)?;
+
+        let unreceived_acks_by_dst = unreceived_acknowledgements_sequences(
+            self.dst_chain(),
+            self.dst_port_id(),
+            dst_channel_id,
+            acks_on_src,
+        )
+        .map_err(LinkError::supervisor)?;
 
         let query_height = opt_query_height.unwrap_or(src_response_height);
 
@@ -932,14 +1102,6 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
             return Ok((events_result.into(), query_height));
         }
 
-        debug!(
-            "packets that have acknowledgments on {}: [{:?}..{:?}] (total={})",
-            self.src_chain().id(),
-            acks_on_src.first(),
-            acks_on_src.last(),
-            acks_on_src.len()
-        );
-
         debug!(
             "ack packets to send out to {} of the ones with acknowledgments on {}: {} (first 10 shown here; total={})",
             self.dst_chain().id(),
@@ -952,9 +1114,9 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
             .query_txs(QueryTxRequest::Packet(QueryPacketEventDataRequest {
                 event_id: WithBlockDataType::WriteAck,
                 source_port_id: self.dst_port_id().clone(),
-                source_channel_id: dst_channel_id.clone(),
+                source_channel_id: *dst_channel_id,
                 destination_port_id: self.src_port_id().clone(),
-                destination_channel_id: src_channel_id.clone(),
+                destination_channel_id: *src_channel_id,
                 sequences,
                 height: query_height,
             }))
@@ -1076,7 +1238,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
 
         let msg = MsgAcknowledgement::new(
             packet,
-            event.ack.clone(),
+            event.ack.clone().into(),
             proofs.clone(),
             self.dst_signer()?,
         );
@@ -1173,7 +1335,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
     fn build_timeout_from_send_packet_event(
         &self,
         event: &SendPacket,
-        dst_info: &StatusResponse,
+        dst_info: &ChainStatus,
     ) -> Result<Option<Any>, LinkError> {
         let packet = event.packet.clone();
         if self
@@ -1191,7 +1353,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
     fn build_recv_or_timeout_from_send_packet_event(
         &self,
         event: &SendPacket,
-        dst_info: &StatusResponse,
+        dst_info: &ChainStatus,
     ) -> Result<(Option<Any>, Option<Any>), LinkError> {
         let timeout = self.build_timeout_from_send_packet_event(event, dst_info)?;
         if timeout.is_some() {
@@ -1209,7 +1371,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
     /// Retains the operational data as pending, and associates it
     /// with one or more transaction hash(es).
     pub fn execute_schedule(&self) -> Result<(), LinkError> {
-        let (src_ods, dst_ods) = self.try_fetch_scheduled_operational_data();
+        let (src_ods, dst_ods) = self.try_fetch_scheduled_operational_data()?;
 
         for od in dst_ods {
             let reply =
@@ -1227,17 +1389,20 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
         Ok(())
     }
 
-    pub fn process_pending_txs(&self) -> RelaySummary {
+    /// Kicks off the process of relaying pending txs to the source and destination chains.
+    ///
+    /// See [`Resubmit::from_clear_interval`] for more info about the `resubmit` parameter.
+    pub fn process_pending_txs(&self, resubmit: Resubmit) -> RelaySummary {
         if !self.confirm_txes {
             return RelaySummary::empty();
         }
 
-        let mut summary_src = self.process_pending_txs_src().unwrap_or_else(|e| {
+        let mut summary_src = self.process_pending_txs_src(resubmit).unwrap_or_else(|e| {
             error!("error processing pending events in source chain: {}", e);
             RelaySummary::empty()
         });
 
-        let summary_dst = self.process_pending_txs_dst().unwrap_or_else(|e| {
+        let summary_dst = self.process_pending_txs_dst(resubmit).unwrap_or_else(|e| {
             error!(
                 "error processing pending events in destination chain: {}",
                 e
@@ -1249,23 +1414,33 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
         summary_src
     }
 
-    fn process_pending_txs_src(&self) -> Result<RelaySummary, LinkError> {
+    fn process_pending_txs_src(&self, resubmit: Resubmit) -> Result<RelaySummary, LinkError> {
+        let do_resubmit = match resubmit {
+            Resubmit::Yes => {
+                Some(|odata| self.relay_from_operational_data::<relay_sender::AsyncSender>(odata))
+            }
+            Resubmit::No => None,
+        };
+
         let res = self
             .pending_txs_src
-            .process_pending(pending::TIMEOUT, |odata| {
-                self.relay_from_operational_data::<relay_sender::AsyncSender>(odata)
-            })?
+            .process_pending(pending::TIMEOUT, self, do_resubmit)?
             .unwrap_or_else(RelaySummary::empty);
 
         Ok(res)
     }
 
-    fn process_pending_txs_dst(&self) -> Result<RelaySummary, LinkError> {
+    fn process_pending_txs_dst(&self, resubmit: Resubmit) -> Result<RelaySummary, LinkError> {
+        let do_resubmit = match resubmit {
+            Resubmit::Yes => {
+                Some(|odata| self.relay_from_operational_data::<relay_sender::AsyncSender>(odata))
+            }
+            Resubmit::No => None,
+        };
+
         let res = self
             .pending_txs_dst
-            .process_pending(pending::TIMEOUT, |odata| {
-                self.relay_from_operational_data::<relay_sender::AsyncSender>(odata)
-            })?
+            .process_pending(pending::TIMEOUT, self, do_resubmit)?
             .unwrap_or_else(RelaySummary::empty);
 
         Ok(res)
@@ -1320,6 +1495,7 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
                                         dst_current_height,
                                         OperationalDataTarget::Source,
                                         &odata.tracking_id,
+                                        self.channel.connection_delay,
                                     )
                                 })
                                 .push(TransitMessage {
@@ -1387,22 +1563,32 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
         }
 
         // Update clients ahead of scheduling the operational data, if the delays are non-zero.
-        if !self.zero_delay() {
-            debug!("connection delay is non-zero: updating client");
+        // If the connection-delay must be taken into account, set the `scheduled_time` to an
+        // instant in the past, i.e. when this client update was first processed (`processed_time`)
+        let scheduled_time = if od.conn_delay_needed() {
+            debug!("connection delay must be taken into account: updating client");
             let target_height = od.proofs_height.increment();
             match od.target {
                 OperationalDataTarget::Source => {
-                    self.update_client_src(target_height, &od.tracking_id)?
+                    let update_height = self.update_client_src(target_height, &od.tracking_id)?;
+                    od.set_update_height(update_height);
+                    self.src_time_at_height(update_height)?
                 }
                 OperationalDataTarget::Destination => {
-                    self.update_client_dst(target_height, &od.tracking_id)?
+                    let update_height = self.update_client_dst(target_height, &od.tracking_id)?;
+                    od.set_update_height(update_height);
+                    self.dst_time_at_height(update_height)?
                 }
-            };
+            }
         } else {
-            debug!("connection delay is zero: client update message will be prepended later");
-        }
+            debug!(
+                "connection delay need not be taken into account: client update message will be \
+            prepended later"
+            );
+            Instant::now()
+        };
 
-        od.scheduled_time = Instant::now();
+        od.set_scheduled_time(scheduled_time);
 
         match od.target {
             OperationalDataTarget::Source => self.src_operational_data.push_back(od),
@@ -1413,88 +1599,51 @@ impl<ChainA: ChainHandle, ChainB: ChainHandle> RelayPath<ChainA, ChainB> {
     }
 
     /// Pulls out the operational elements with elapsed delay period and that can
-    /// now be processed. Does not block: if no OD fulfilled the delay period (or none is
-    /// scheduled), returns immediately with `vec![]`.
+    /// now be processed.
     fn try_fetch_scheduled_operational_data(
         &self,
-    ) -> (VecDeque<OperationalData>, VecDeque<OperationalData>) {
+    ) -> Result<(VecDeque<OperationalData>, VecDeque<OperationalData>), LinkError> {
         // Extracts elements from a Vec when the predicate returns true.
         // The mutable vector is then updated to the remaining unextracted elements.
         fn partition<T>(
             queue: VecDeque<T>,
-            pred: impl Fn(&T) -> bool,
-        ) -> (VecDeque<T>, VecDeque<T>) {
+            pred: impl Fn(&T) -> Result<bool, LinkError>,
+        ) -> Result<(VecDeque<T>, VecDeque<T>), LinkError> {
             let mut true_res = VecDeque::new();
             let mut false_res = VecDeque::new();
 
             for e in queue.into_iter() {
-                if pred(&e) {
+                if pred(&e)? {
                     true_res.push_back(e);
                 } else {
                     false_res.push_back(e);
                 }
             }
 
-            (true_res, false_res)
+            Ok((true_res, false_res))
         }
 
-        let connection_delay = self.channel.connection_delay;
         let (elapsed_src_ods, unelapsed_src_ods) =
             partition(self.src_operational_data.take(), |op| {
-                op.scheduled_time.elapsed() > connection_delay
-            });
-
-        self.src_operational_data.replace(unelapsed_src_ods);
+                op.has_conn_delay_elapsed(
+                    &|| self.src_time_latest(),
+                    &|| self.src_max_block_time(),
+                    &|| self.src_latest_height(),
+                )
+            })?;
 
         let (elapsed_dst_ods, unelapsed_dst_ods) =
             partition(self.dst_operational_data.take(), |op| {
-                op.scheduled_time.elapsed() > connection_delay
-            });
+                op.has_conn_delay_elapsed(
+                    &|| self.dst_time_latest(),
+                    &|| self.dst_max_block_time(),
+                    &|| self.dst_latest_height(),
+                )
+            })?;
 
+        self.src_operational_data.replace(unelapsed_src_ods);
         self.dst_operational_data.replace(unelapsed_dst_ods);
-
-        (elapsed_src_ods, elapsed_dst_ods)
-    }
-
-    /// Fetches an operational data that has fulfilled its predefined delay period. May _block_
-    /// waiting for the delay period to pass.
-    /// Returns `None` if there is no operational data scheduled.
-    pub(crate) fn fetch_scheduled_operational_data(&self) -> Option<OperationalData> {
-        let odata = self
-            .src_operational_data
-            .pop_front()
-            .or_else(|| self.dst_operational_data.pop_front());
-
-        if let Some(odata) = odata {
-            // Check if the delay period did not completely elapse
-            let delay_left = self
-                .channel
-                .connection_delay
-                .checked_sub(odata.scheduled_time.elapsed());
-
-            match delay_left {
-                None => info!(
-                    "ready to fetch a scheduled op. data with batch of size {} targeting {}",
-                    odata.batch.len(),
-                    odata.target,
-                ),
-                Some(delay_left) => {
-                    info!(
-                        "waiting ({:?} left) for a scheduled op. data with batch of size {} targeting {}",
-                        delay_left,
-                        odata.batch.len(),
-                        odata.target,
-                    );
-
-                    // Wait until the delay period passes
-                    thread::sleep(delay_left);
-                }
-            }
-
-            Some(odata)
-        } else {
-            None
-        }
+        Ok((elapsed_src_ods, elapsed_dst_ods))
     }
 
     fn restore_src_client(&self) -> ForeignClient<ChainA, ChainB> {
