@@ -375,7 +375,7 @@ impl Object {
         Ok(Channel {
             dst_chain_id,
             src_chain_id: src_chain.id(),
-            src_channel_id: channel_id.clone(),
+            src_channel_id: *channel_id,
             src_port_id: attributes.port_id().clone(),
         }
         .into())
@@ -396,7 +396,7 @@ impl Object {
         Ok(Packet {
             dst_chain_id,
             src_chain_id: src_chain.id(),
-            src_channel_id: e.packet.source_channel.clone(),
+            src_channel_id: e.packet.source_channel,
             src_port_id: e.packet.source_port.clone(),
         }
         .into())
@@ -417,7 +417,7 @@ impl Object {
         Ok(Packet {
             dst_chain_id,
             src_chain_id: src_chain.id(),
-            src_channel_id: e.packet.destination_channel.clone(),
+            src_channel_id: e.packet.destination_channel,
             src_port_id: e.packet.destination_port.clone(),
         }
         .into())
@@ -438,7 +438,7 @@ impl Object {
         Ok(Packet {
             dst_chain_id,
             src_chain_id: src_chain.id(),
-            src_channel_id: e.src_channel_id().clone(),
+            src_channel_id: *e.src_channel_id(),
             src_port_id: e.src_port_id().clone(),
         }
         .into())
@@ -455,7 +455,7 @@ impl Object {
         Ok(Packet {
             dst_chain_id,
             src_chain_id: src_chain.id(),
-            src_channel_id: e.channel_id().clone(),
+            src_channel_id: *e.channel_id(),
             src_port_id: e.port_id().clone(),
         }
         .into())
