@@ -15,6 +15,10 @@ define_error! {
         UnknownSdk
             { code: u32 }
             |e| { format!("unknown SDK error: {}", e.code) },
+
+        OutOfGas
+            { code: u32 }
+            |_| { "the price configuration for this chain may be too low! please check the `gas_price.price` Hermes config.toml".to_string() },
     }
 }
 
@@ -166,3 +170,17 @@ pub fn sdk_error_from_tx_result(result: &TxResult) -> SdkError {
         }
     }
 }
+
+/// Converts error codes originating from `broadcast_tx_sync` responses
+/// into IBC relayer domain-type errors.
+/// See [`tendermint_rpc::endpoint::broadcast::tx_sync::Response`].
+/// Cf: <https://github.com/cosmos/cosmos-sdk/blob/v0.42.10/types/errors/errors.go>
+pub fn sdk_error_from_tx_sync_error_code(code: u32) -> SdkError {
+    match code {
+        // The primary reason (we know of) causing broadcast_tx_sync to fail
+        // is due to "out of gas" errors. These are unrecoverable at the moment
+        // on the Hermes side. We'll inform the user to check for misconfig.
+        11 => SdkError::out_of_gas(code),
+        _ => SdkError::unknown_sdk(code),
+    }
+}
