@@ -21,7 +21,7 @@ use super::{
     Error,
 };
 
-/// A context for spawning workers within the [`crate::supervisor::Supervisor`].
+/// A context for spawning workers within the supervisor.
 pub struct SpawnContext<'a, Chain: ChainHandle> {
     config: &'a Config,
     registry: &'a mut Registry<Chain>,
@@ -186,7 +186,7 @@ impl<'a, Chain: ChainHandle> SpawnContext<'a, Chain> {
         }
     }
 
-    /// Spawns all the [`Worker`](crate::worker::Worker)s that will
+    /// Spawns all the [`WorkerHandle`](crate::worker::WorkerHandle)s that will
     /// handle a given channel for a given source chain.
     pub fn spawn_workers_for_channel(
         &mut self,
@@ -259,7 +259,7 @@ impl<'a, Chain: ChainHandle> SpawnContext<'a, Chain> {
                     let path_object = Object::Packet(Packet {
                         dst_chain_id: counterparty_chain.id(),
                         src_chain_id: chain.id(),
-                        src_channel_id: channel_scan.id().clone(),
+                        src_channel_id: *channel_scan.id(),
                         src_port_id: channel_scan.channel.port_id.clone(),
                     });
 
@@ -283,7 +283,7 @@ impl<'a, Chain: ChainHandle> SpawnContext<'a, Chain> {
             let channel_object = Object::Channel(Channel {
                 dst_chain_id: counterparty_chain.id(),
                 src_chain_id: chain.id(),
-                src_channel_id: channel_scan.id().clone(),
+                src_channel_id: *channel_scan.id(),
                 src_port_id: channel_scan.channel.port_id,
             });
 
