@@ -1,6 +1,6 @@
 // If the `telemetry` feature is enabled, re-export the `ibc-telemetry` state.
 #[cfg(feature = "telemetry")]
-pub type Telemetry = std::sync::Arc<ibc_telemetry::TelemetryState>;
+pub type Telemetry = alloc::sync::Arc<ibc_telemetry::TelemetryState>;
 
 // Otherwise, define and export a dummy type.
 #[cfg(not(feature = "telemetry"))]
@@ -14,17 +14,30 @@ pub type Telemetry = TelemetryDisabled;
 /// only if the `telemetry` feature is enabled.
 /// Otherwise, it compiles to a no-op.
 ///
-/// ## Note
-/// Equivalent to `#[cfg(feature = "telemetry")]`, but
-/// should be preferred over the latter.
+/// The macro can be used in two ways:
 ///
-/// ## Example
+/// a) By passing it an arbitrary expression as its single argument.
 ///
 /// ```rust,ignore
-/// telemetry!(self.telemetry.tx_count(1));
+/// telemetry!(ibc_telemetry::global().tx_count(chain.id(), 1));
 /// ```
+///
+/// b) In the common case where one wants to update a metric,
+///    the macro accepts the metric's name, followed by its arguments.
+///
+/// ```rust,ignore
+/// telemetry!(tx_count, chain.id(), 1);
+/// ```
+///
 #[macro_export]
 macro_rules! telemetry {
+    ($id:ident, $($args:expr),* $(,)*) => {
+        #[cfg(feature = "telemetry")]
+        {
+            ::ibc_telemetry::global().$id($($args),*);
+        }
+    };
+
     ($e:expr) => {
         #[cfg(feature = "telemetry")]
         {
