@@ -1,11 +1,15 @@
 use core::time::Duration;
-use crossbeam_channel::Receiver;
-use ibc::Height;
 use std::sync::{Arc, Mutex};
-use tracing::{error, error_span, trace};
+
+use crossbeam_channel::Receiver;
+use tracing::{error, error_span, trace, warn};
+
+use ibc::core::ics04_channel::channel::Order;
+use ibc::Height;
 
 use crate::chain::handle::ChainHandle;
 use crate::foreign_client::HasExpiredOrFrozenError;
+use crate::link::Resubmit;
 use crate::link::{error::LinkError, Link};
 use crate::object::Packet;
 use crate::telemetry;
@@ -17,18 +21,25 @@ use super::error::RunError;
 use super::WorkerCmd;
 
 /// Whether or not to clear pending packets at this `step` for the given height.
-/// Packets are cleared on the first iteration if `clear_on_start` is true.
+/// Packets are cleared on the first iteration if this is an ordered channel, or
+/// if `clear_on_start` is true.
 /// Subsequently, packets are cleared only if `clear_interval` is not `0` and
 /// if we have reached the interval.
 fn should_clear_packets(
     is_first_run: &mut bool,
     clear_on_start: bool,
+    channel_ordering: Order,
     clear_interval: u64,
     height: Height,
 ) -> bool {
     if *is_first_run {
         *is_first_run = false;
-        clear_on_start
+        if channel_ordering == Order::Ordered {
+            warn!("ordered channel: will clear packets because this is the first run");
+            true
+        } else {
+            clear_on_start
+        }
     } else {
         clear_interval != 0 && height.revision_height % clear_interval == 0
     }
@@ -42,10 +53,13 @@ fn handle_link_error_in_task(e: LinkError) -> TaskError<RunError> {
     }
 }
 
+/// Spawns a packet worker task in the background that handles the work of
+/// processing pending txs between `ChainA` and `ChainB`.
 pub fn spawn_packet_worker<ChainA: ChainHandle, ChainB: ChainHandle>(
     path: Packet,
     // Mutex is used to prevent race condition between the packet workers
     link: Arc<Mutex<Link<ChainA, ChainB>>>,
+    resubmit: Resubmit,
 ) -> TaskHandle {
     let span = {
         let relay_path = &link.lock().unwrap().a_to_b;
@@ -69,10 +83,10 @@ pub fn spawn_packet_worker<ChainA: ChainHandle, ChainB: ChainHandle>(
             .execute_schedule()
             .map_err(handle_link_error_in_task)?;
 
-        let summary = relay_path.process_pending_txs();
+        let summary = relay_path.process_pending_txs(resubmit);
 
         if !summary.is_empty() {
-            trace!("Packet worker produced relay summary: {:?}", summary);
+            trace!("packet worker produced relay summary: {}", summary);
         }
 
         telemetry!(packet_metrics(&path, &summary));
@@ -138,7 +152,7 @@ fn handle_packet_cmd<ChainA: ChainHandle, ChainB: ChainHandle>(
     cmd: WorkerCmd,
     index: u64,
 ) -> RetryResult<(), u64> {
-    trace!("handling command {:?}", cmd);
+    trace!("handling command {}", cmd);
     let result = match cmd {
         WorkerCmd::IbcEvents { batch } => link.a_to_b.update_schedule(batch),
 
@@ -151,7 +165,13 @@ fn handle_packet_cmd<ChainA: ChainHandle, ChainB: ChainHandle>(
             // Decide if packet clearing should be scheduled.
             // Packet clearing may happen once at start,
             // and then at predefined block intervals.
-            if should_clear_packets(is_first_run, clear_on_start, clear_interval, height) {
+            if should_clear_packets(
+                is_first_run,
+                clear_on_start,
+                link.a_to_b.channel().ordering,
+                clear_interval,
+                height,
+            ) {
                 link.a_to_b.schedule_packet_clearing(Some(height))
             } else {
                 Ok(())
@@ -202,7 +222,8 @@ fn handle_packet_cmd<ChainA: ChainHandle, ChainB: ChainHandle>(
         }
     }
 
-    let summary = link.a_to_b.process_pending_txs();
+    let resubmit = Resubmit::from_clear_interval(clear_interval);
+    let summary = link.a_to_b.process_pending_txs(resubmit);
 
     if !summary.is_empty() {
         trace!("produced relay summary: {:?}", summary);
